/*************************************************************************/
/*************************************************************************/
/*                                                                       */
/*  File Name : VRProblem.cc                                             */
/*                                                                       */
/*       Date : May  5, 1999                                             */
/* Updated at : May  5, 1999                                             */
/*                                                                       */
/*     Author : H. Timucin OZDEMIR                                       */
/*                                                                       */
/*************************************************************************/
/*************************************************************************/
/* Representation of the Vehicle Routing Problem                         */
/*************************************************************************/

#include <iomanip.h>
#include <iostream.h>
#include <string.h>
#include <stdio.h>

#include <deffunc.h>   // ../commons
#include <myQueue.h>   // ../TOOLS

#include "VRProblem.h"

/*************************************************************************/
/*************************************************************************/

VRProblem::VRProblem()
{  
   myCustomers = new CustomerList(100);
   myDepots = new DepotList(this,10);
   maxVehicleCapacity=200;
   //EdgeGraph  *myEdgeGraph;

}// end of constructor

/*************************************************************************/
/*************************************************************************/

VRProblem::VRProblem(const VRProblem& RHS)
{  

}// end of COPY constructor

/*************************************************************************/
/*************************************************************************/

VRProblem::~VRProblem() {  dispose(); }

/*************************************************************************/
// returns the number of customers.
/*************************************************************************/

unsigned VRProblem::getNumberOfCustomers() const 
{ return (myCustomers==NULL?0:myCustomers->getSize()); }

/*************************************************************************/
// returns the list of depots.
/*************************************************************************/

DepotList* VRProblem::getDepotList() const 
{ return myDepots; }

/*************************************************************************/
// return 0: if indx is invalid.
/*************************************************************************/

VRProblem::getDepot(unsigned indx, Customer& ret) const 
{ int retVal=1;
  customerId id;
  if(indx>=0&&indx<myDepots->getSize()&& myDepots->getDepot(indx,id))
       retVal=getNodeWithNodeId(id,ret);
  else retVal=0;
  return retVal;
}// end of getDepot()

/*************************************************************************/
// return 0: if indx is invalid.
/*************************************************************************/

VRProblem::isDepot(customerId indx) const 
{ int retVal=0, i=0;
  customerId d;
  while(!retVal&&i<myDepots->getSize())
    if(myDepots->getDepot(i,d)&&d==indx) retVal=1;
    else i++;
  return retVal;
}// end of isDepot()

/*************************************************************************/
/*************************************************************************/

VRProblem::getNodeWithNodeId(customerId indx, Customer& ret) const 
{ int retVal=0, i=0;
  while(!retVal&&i<myCustomers->getSize())
    if(myCustomers->getCustomer(i,ret)&&ret.getNodeNumber()==indx) retVal=1;
    else i++;
  return retVal;
}// end of getNodeWithNodeId()

/*************************************************************************/
/** returns the index of customer specified with customerId */
/*************************************************************************/

VRProblem::getNodeIndex(customerId indx) const 
{ int retVal=0, i=0;
  Customer ret;
  while(!retVal&&i<myCustomers->getSize())
    if(myCustomers->getCustomer(i,ret)&&ret.getNodeNumber()==indx) retVal=1;
    else i++;
  return (retVal?i:-1);
}// end of getNodeIndex()

/*************************************************************************/
/*************************************************************************/

void VRProblem::addDepot(customerId& newEle) { myDepots->addDepot(newEle); }

/*************************************************************************/
// returns the list of customers.
/*************************************************************************/

CustomerListPtr VRProblem::getCustomerList() const 
{ return myCustomers; }

/*************************************************************************/
// return 0: if indx is invalid.
/*************************************************************************/

VRProblem::getCustomer(unsigned indx, Customer& ret) const 
{ int retVal=1;
  if(indx>=0&&indx<myCustomers->getSize()) myCustomers->getCustomer(indx,ret);
  else retVal=0;
  if(!retVal) cout << " customer " << indx << "/" << myCustomers->getSize() 
                   << " does not exist! " << endl;
  return retVal;
}// end of getCustomer()

/*************************************************************************/
/*************************************************************************/
void VRProblem::addCustomer(Customer& newEle) 
{ myCustomers->addCustomer(newEle); }

/*************************************************************************/
// returns the capacity of a vehicle
/*************************************************************************/

vrpcapacity VRProblem::getMaximumCapacity() const 
{ return test.getVehicleCapacity(); }

/*************************************************************************/
// returns the total demand 
/*************************************************************************/

vrpcapacity VRProblem::getTotalOfDemans() const 
{ vrpcapacity totalDemands=(vrpcapacity)0;
  int i=0;
  Customer cus;
  for(i=0;i<myCustomers->getSize();i++) 
     if(myCustomers->getCustomer(i,cus)) totalDemands+=cus.getDemand();
  return totalDemands;
}// end of getTotalOfDemans()

/*************************************************************************/
// assignment operator
/*************************************************************************/

VRProblem& VRProblem::operator=(const VRProblem& RHS) 
{  dispose();
   copyFrom(RHS); 
   return *this;
}

/*************************************************************************/
/*************************************************************************/

VRProblem::operator==(const VRProblem& RHS) 
{  int retVal = 0;
   return retVal;
}// end of operator==()

/*************************************************************************/
/*************************************************************************/

ostream& operator<<(ostream& os, const VRProblem& RHS)
{  
   os << " Vehicle Routing Problem " << endl;
   os << " Number of Depots ...............: " << RHS.myDepots->getSize() << endl;
   os << " Number of Customer .............: " << RHS.getNumberOfCustomers() << endl;
   os << " Maximum capacity of a vehicle ..: " << RHS.getMaximumCapacity() << endl;
   os << " Total of customer Demans .......: " << RHS.getTotalOfDemans() << endl;
   return os;
}// end of operator<<()

/*************************************************************************/
/*************************************************************************/
void VRProblem::dispose()
{  delete myCustomers;
   delete myDepots;
   //delete myEdgeGraph;
}// end of dispose()

/*************************************************************************/
/*************************************************************************/
void VRProblem::copyFrom(const VRProblem& RHS) 
{  
   maxVehicleCapacity = RHS.maxVehicleCapacity;
}

